@@ -3,31 +3,79 @@
 //!
 //! `chainerror` has no dependencies!
 //!
-//! `chainerror` uses `.source()` of `std::error::Error` along with `line()!` and `file()!` to provide a nice debug error backtrace.
+//! `chainerror` uses `.source()` of `std::error::Error` along with `#[track_caller]` and `Location` to provide a nice debug error backtrace.
 //! It encapsulates all types, which have `Display + Debug` and can store the error cause internally.
 //!
 //! Along with the `ChainError<T>` struct, `chainerror` comes with some useful helper macros to save a lot of typing.
 //!
-//! ## Features
+//! Debug information is worth it!
 //!
-//! `no-fileline`
-//! : completely turn off storing filename and line
+//! ## Features
 //!
 //! `display-cause`
 //! : turn on printing a backtrace of the errors in `Display`
 //!
-//! `no-debug-cause`
-//! : turn off printing a backtrace of the errors in `Debug`
-//!
-//!
 //! # Tutorial
 //!
 //! Read the [Tutorial](https://haraldh.github.io/chainerror/tutorial1.html)
 //!
 //! # Examples
 //!
+//! examples/example.rs:
+//! ```rust,ignore
+//! // […]
+//! fn main() {
+//!    if let Err(e) = func1() {
+//!        eprintln!("\nDebug Error {{:?}}:\n{:?}", e);
+//!        eprintln!("\nAlternative Debug Error {{:#?}}:\n{:#?}\n", e);
+//!        // […]
+//!   }
+//! }
+//! ```
+//!
+//! ```console
+//! $ cargo run -q --example example
+//! Debug Error {:?}:
+//! examples/example.rs:46:13: func1 error calling func2
+//! Caused by:
+//! examples/example.rs:21:13: Func2Error(func2 error: calling func3)
+//! Caused by:
+//! examples/example.rs:14:18: Error reading 'foo.txt'
+//! Caused by:
+//! Kind(NotFound)
+//!
+//! Alternative Debug Error {:#?}:
+//! ChainError<example::Func1Error> {
+//!     occurrence: Some(
+//!         "examples/example.rs:46:13",
+//!     ),
+//!     kind: func1 error calling func2,
+//!     source: Some(
+//!         ChainError<example::Func2Error> {
+//!             occurrence: Some(
+//!                 "examples/example.rs:21:13",
+//!             ),
+//!             kind: Func2Error(func2 error: calling func3),
+//!             source: Some(
+//!                 ChainError<alloc::string::String> {
+//!                     occurrence: Some(
+//!                         "examples/example.rs:14:18",
+//!                    ),
+//!                    kind: "Error reading \'foo.txt\'",
+//!                     source: Some(
+//!                         Kind(
+//!                             NotFound,
+//!                         ),
+//!                     ),
+//!                 },
+//!             ),
+//!         },
+//!     ),
+//! }
+//! ```
+//!
 //! ```rust
-//! use chainerror::*;
+//! use chainerror::prelude::v1::*;
 //! use std::error::Error;
 //! use std::io;
 //! use std::result::Result;
@@ -39,37 +87,36 @@
 //!
 //! fn func2() -> Result<(), Box<dyn Error + Send + Sync>> {
 //!     let filename = "foo.txt";
-//!     do_some_io().map_err(mstrerr!("Error reading '{}'", filename))?;
+//!     do_some_io().context(format!("Error reading '{}'", filename))?;
 //!     Ok(())
 //! }
 //!
 //! fn func1() -> Result<(), Box<dyn Error + Send + Sync>> {
-//!     func2().map_err(mstrerr!("func1 error"))?;
+//!     func2().context("func1 error")?;
 //!     Ok(())
 //! }
 //!
-//! fn main() {
-//!     if let Err(e) = func1() {
-//!         #[cfg(not(windows))]
-//!         assert_eq!(
-//!             format!("\n{:?}\n", e), r#"
-//! src/lib.rs:20: func1 error
+//! if let Err(e) = func1() {
+//!     #[cfg(not(windows))]
+//!     assert_eq!(
+//!         format!("\n{:?}\n", e),
+//!         r#"
+//! src/lib.rs:21:13: func1 error
 //! Caused by:
-//! src/lib.rs:15: Error reading 'foo.txt'
+//! src/lib.rs:16:18: Error reading 'foo.txt'
 //! Caused by:
 //! Kind(NotFound)
 //! "#
-//!         );
-//!     }
+//!     );
+//! }
 //! #    else {
 //! #        unreachable!();
 //! #    }
-//! }
 //! ```
 //!
 //!
 //! ```rust
-//! use chainerror::*;
+//! use chainerror::prelude::v1::*;
 //! use std::error::Error;
 //! use std::io;
 //! use std::result::Result;
@@ -81,14 +128,14 @@
 //!
 //! fn func3() -> Result<(), Box<dyn Error + Send + Sync>> {
 //!     let filename = "foo.txt";
-//!     do_some_io().map_err(mstrerr!("Error reading '{}'", filename))?;
+//!     do_some_io().context(format!("Error reading '{}'", filename))?;
 //!     Ok(())
 //! }
 //!
-//! derive_str_cherr!(Func2Error);
+//! derive_str_context!(Func2Error);
 //!
 //! fn func2() -> ChainResult<(), Func2Error> {
-//!     func3().map_err(mstrerr!(Func2Error, "func2 error: calling func3"))?;
+//!     func3().context(Func2Error("func2 error: calling func3".into()))?;
 //!     Ok(())
 //! }
 //!
@@ -113,91 +160,79 @@
 //! }
 //!
 //! fn func1() -> ChainResult<(), Func1Error> {
-//!     func2().map_err(|e| cherr!(e, Func1Error::Func2))?;
+//!     func2().context(Func1Error::Func2)?;
 //!     let filename = String::from("bar.txt");
-//!     do_some_io().map_err(|e| cherr!(e, Func1Error::IO(filename)))?;
+//!     do_some_io().context(Func1Error::IO(filename))?;
 //!     Ok(())
 //! }
 //!
-//! fn main() {
-//!     if let Err(e) = func1() {
-//!         assert!(
-//!             match e.kind() {
-//!                 Func1Error::Func2 => {
-//!                     eprintln!("Main Error Report: func1 error calling func2");
-//!                     true
-//!                 }
-//!                 Func1Error::IO(filename) => {
-//!                     eprintln!("Main Error Report: func1 error reading '{}'", filename);
-//!                     false
-//!                 }
-//!             }
-//!         );
-//!
-//!         assert!(e.find_chain_cause::<Func2Error>().is_some());
-//!
-//!         if let Some(e) = e.find_chain_cause::<Func2Error>() {
-//!             eprintln!("\nError reported by Func2Error: {}", e)
+//! if let Err(e) = func1() {
+//!     assert!(match e.kind() {
+//!         Func1Error::Func2 => {
+//!             eprintln!("Main Error Report: func1 error calling func2");
+//!             true
 //!         }
-//!
-//!
-//!         assert!(e.root_cause().is_some());
-//!
-//!         if let Some(e) = e.root_cause() {
-//!             let io_error = e.downcast_ref::<io::Error>().unwrap();
-//!             eprintln!("\nThe root cause was: std::io::Error: {:#?}", io_error);
+//!         Func1Error::IO(filename) => {
+//!             eprintln!("Main Error Report: func1 error reading '{}'", filename);
+//!             false
 //!         }
+//!     });
 //!
-//!         #[cfg(not(windows))]
-//!         assert_eq!(
-//!             format!("\n{:?}\n", e), r#"
-//! src/lib.rs:47: func1 error calling func2
+//!     assert!(e.find_chain_cause::<Func2Error>().is_some());
+//!
+//!     if let Some(e) = e.find_chain_cause::<Func2Error>() {
+//!         eprintln!("\nError reported by Func2Error: {}", e)
+//!     }
+//!
+//!     assert!(e.root_cause().is_some());
+//!
+//!     if let Some(e) = e.root_cause() {
+//!         let io_error = e.downcast_ref::<io::Error>().unwrap();
+//!         eprintln!("\nThe root cause was: std::io::Error: {:#?}", io_error);
+//!     }
+//!
+//!     #[cfg(not(windows))]
+//!     assert_eq!(
+//!         format!("\n{:?}\n", e),
+//!         r#"
+//! src/lib.rs:48:13: func1 error calling func2
 //! Caused by:
-//! src/lib.rs:22: Func2Error(func2 error: calling func3)
+//! src/lib.rs:23:13: Func2Error(func2 error: calling func3)
 //! Caused by:
-//! src/lib.rs:15: Error reading 'foo.txt'
+//! src/lib.rs:16:18: Error reading 'foo.txt'
 //! Caused by:
 //! Kind(NotFound)
 //! "#
-//!         );
-//!     }
+//!     );
+//! }
 //! #    else {
 //! #        unreachable!();
 //! #    }
-//! }
 //! ```
 
-#![deny(
-    warnings,
-    absolute_paths_not_starting_with_crate,
-    deprecated_in_future,
-    keyword_idents,
-    macro_use_extern_crate,
-    missing_debug_implementations,
-    trivial_numeric_casts,
-    unused_extern_crates,
-    unused_import_braces,
-    unused_qualifications,
-    unused_results,
-    unused_labels,
-    unused_lifetimes,
-    unstable_features,
-    unreachable_pub,
-    future_incompatible,
-    missing_copy_implementations,
-    missing_doc_code_examples,
-    rust_2018_idioms,
-    rust_2018_compatibility
-)]
+#![deny(clippy::all)]
+#![deny(clippy::integer_arithmetic)]
+#![deny(missing_docs)]
 
 use std::any::TypeId;
 use std::error::Error;
 use std::fmt::{Debug, Display, Formatter, Result};
+use std::panic::Location;
+
+pub mod prelude {
+    //! convenience prelude
+    pub mod v1 {
+        //! convenience prelude
+        pub use super::super::ChainErrorDown as _;
+        pub use super::super::ResultTrait as _;
+        pub use super::super::{ChainError, ChainResult};
+        pub use crate::{derive_err_kind, derive_str_context};
+    }
+}
 
 /// chains an inner error kind `T` with a causing error
 pub struct ChainError<T> {
-    #[cfg(not(feature = "no-fileline"))]
-    occurrence: Option<&'static str>,
+    occurrence: Option<String>,
     kind: T,
     error_cause: Option<Box<dyn Error + 'static + Send + Sync>>,
 }
@@ -206,13 +241,12 @@ pub struct ChainError<T> {
 pub type ChainResult<O, E> = std::result::Result<O, ChainError<E>>;
 
 impl<T: 'static + Display + Debug> ChainError<T> {
-    #[cfg(not(feature = "no-fileline"))]
-    /// Use the `cherr!()` or `mstrerr!()` macro instead of calling this directly
+    /// Use the `context()` or `map_context()` Result methods instead of calling this directly
     #[inline]
     pub fn new(
         kind: T,
         error_cause: Option<Box<dyn Error + 'static + Send + Sync>>,
-        occurrence: Option<&'static str>,
+        occurrence: Option<String>,
     ) -> Self {
         Self {
             occurrence,
@@ -221,17 +255,6 @@ impl<T: 'static + Display + Debug> ChainError<T> {
         }
     }
 
-    #[cfg(feature = "no-fileline")]
-    /// Use the `cherr!()` or `mstrerr!()` macro instead of calling this directly
-    #[inline]
-    pub fn new(
-        kind: T,
-        error_cause: Option<Box<dyn Error + 'static + Send + Sync>>,
-        _occurrence: Option<&'static str>,
-    ) -> Self {
-        Self { kind, error_cause }
-    }
-
     /// return the root cause of the error chain, if any exists
     pub fn root_cause(&self) -> Option<&(dyn Error + 'static)> {
         self.iter().last()
@@ -242,7 +265,7 @@ impl<T: 'static + Display + Debug> ChainError<T> {
     /// # Examples
     ///
     /// ```rust
-    /// use chainerror::*;
+    /// use chainerror::prelude::v1::*;
     /// use std::error::Error;
     /// use std::io;
     ///
@@ -251,37 +274,34 @@ impl<T: 'static + Display + Debug> ChainError<T> {
     ///     Ok(())
     /// }
     ///
-    /// derive_str_cherr!(Func2Error);
+    /// derive_str_context!(Func2Error);
     ///
     /// fn func2() -> Result<(), Box<dyn Error + Send + Sync>> {
     ///     let filename = "foo.txt";
-    ///     do_some_io().map_err(mstrerr!(Func2Error, "Error reading '{}'", filename))?;
+    ///     do_some_io().context(Func2Error(format!("Error reading '{}'", filename)))?;
     ///     Ok(())
     /// }
     ///
-    /// derive_str_cherr!(Func1Error);
+    /// derive_str_context!(Func1Error);
     ///
     /// fn func1() -> Result<(), Box<dyn Error + Send + Sync>> {
-    ///     func2().map_err(mstrerr!(Func1Error, "func1 error"))?;
+    ///     func2().context(Func1Error("func1 error".into()))?;
     ///     Ok(())
     /// }
     ///
-    /// fn main() {
-    ///     if let Err(e) = func1() {
-    ///         if let Some(f1err) = e.downcast_chain_ref::<Func1Error>() {
+    /// if let Err(e) = func1() {
+    ///     if let Some(f1err) = e.downcast_chain_ref::<Func1Error>() {
+    ///         assert!(f1err.find_cause::<io::Error>().is_some());
     ///
-    ///             assert!(f1err.find_cause::<io::Error>().is_some());
-    ///
-    ///             assert!(f1err.find_chain_cause::<Func2Error>().is_some());
-    ///         }
+    ///         assert!(f1err.find_chain_cause::<Func2Error>().is_some());
+    ///     }
     /// #        else {
     /// #            panic!();
     /// #        }
-    ///     }
+    /// }
     /// #    else {
     /// #         unreachable!();
     /// #    }
-    /// }
     /// ```
     #[inline]
     pub fn find_cause<U: Error + 'static>(&self) -> Option<&U> {
@@ -295,8 +315,8 @@ impl<T: 'static + Display + Debug> ChainError<T> {
     /// # Examples
     ///
     /// ```rust
-    /// # use chainerror::*;
-    /// # derive_str_cherr!(FooError);
+    /// # use chainerror::prelude::v1::*;
+    /// # derive_str_context!(FooError);
     /// # let err = ChainError::new(String::new(), None, None);
     /// // Instead of writing
     /// err.find_cause::<ChainError<FooError>>();
@@ -318,8 +338,8 @@ impl<T: 'static + Display + Debug> ChainError<T> {
     /// # Examples
     ///
     /// ```rust
-    /// # use chainerror::*;
-    /// # derive_str_cherr!(FooErrorKind);
+    /// # use chainerror::prelude::v1::*;
+    /// # derive_str_context!(FooErrorKind);
     /// # let err = ChainError::new(String::new(), None, None);
     /// // Instead of writing
     /// err.find_cause::<ChainError<FooErrorKind>>();
@@ -347,7 +367,7 @@ impl<T: 'static + Display + Debug> ChainError<T> {
     /// # Examples
     ///
     /// ```rust
-    /// use chainerror::*;
+    /// use chainerror::prelude::v1::*;
     /// use std::error::Error;
     /// use std::io;
     ///
@@ -356,11 +376,11 @@ impl<T: 'static + Display + Debug> ChainError<T> {
     ///     Ok(())
     /// }
     ///
-    /// derive_str_cherr!(Func2Error);
+    /// derive_str_context!(Func2Error);
     ///
     /// fn func2() -> Result<(), Box<dyn Error + Send + Sync>> {
     ///     let filename = "foo.txt";
-    ///     do_some_io().map_err(mstrerr!(Func2Error, "Error reading '{}'", filename))?;
+    ///     do_some_io().context(Func2Error(format!("Error reading '{}'", filename)))?;
     ///     Ok(())
     /// }
     ///
@@ -381,22 +401,20 @@ impl<T: 'static + Display + Debug> ChainError<T> {
     /// # }
     ///
     /// fn func1() -> ChainResult<(), Func1ErrorKind> {
-    ///     func2().map_err(|e| cherr!(e, Func1ErrorKind::Func2))?;
-    ///     do_some_io().map_err(|e| cherr!(e, Func1ErrorKind::IO("bar.txt".into())))?;
+    ///     func2().context(Func1ErrorKind::Func2)?;
+    ///     do_some_io().context(Func1ErrorKind::IO("bar.txt".into()))?;
     ///     Ok(())
     /// }
     ///
-    /// fn main() {
-    ///     if let Err(e) = func1() {
-    ///         match e.kind() {
-    ///             Func1ErrorKind::Func2 => {}
-    ///             Func1ErrorKind::IO(filename) => panic!(),
-    ///         }
+    /// if let Err(e) = func1() {
+    ///     match e.kind() {
+    ///         Func1ErrorKind::Func2 => {}
+    ///         Func1ErrorKind::IO(filename) => panic!(),
     ///     }
+    /// }
     /// #    else {
     /// #         unreachable!();
     /// #    }
-    /// }
     /// ```
     #[inline]
     pub fn kind(&self) -> &T {
@@ -406,8 +424,6 @@ impl<T: 'static + Display + Debug> ChainError<T> {
     /// Returns an Iterator over all error causes/sources
     ///
     /// # Example
-    ///
-    ///
     #[inline]
     pub fn iter(&self) -> impl Iterator<Item = &(dyn Error + 'static)> {
         ErrorIter {
@@ -416,7 +432,60 @@ impl<T: 'static + Display + Debug> ChainError<T> {
     }
 }
 
-struct ErrorIter<'a> {
+/// Convenience methods for `Result<>` to turn the error into a decorated ChainError
+pub trait ResultTrait<O, E: Into<Box<dyn Error + 'static + Send + Sync>>> {
+    /// Decorate the error with a `kind` of type `T` and the source `Location`
+    fn context<T: 'static + Display + Debug>(
+        self,
+        kind: T,
+    ) -> std::result::Result<O, ChainError<T>>;
+
+    /// Decorate the `error` with a `kind` of type `T` produced with a `FnOnce(&error)` and the source `Location`
+    fn map_context<T: 'static + Display + Debug, F: FnOnce(&E) -> T>(
+        self,
+        op: F,
+    ) -> std::result::Result<O, ChainError<T>>;
+}
+
+impl<O, E: Into<Box<dyn Error + 'static + Send + Sync>>> ResultTrait<O, E>
+    for std::result::Result<O, E>
+{
+    #[track_caller]
+    fn context<T: 'static + Display + Debug>(
+        self,
+        kind: T,
+    ) -> std::result::Result<O, ChainError<T>> {
+        match self {
+            Ok(t) => Ok(t),
+            Err(error_cause) => Err(ChainError::new(
+                kind,
+                Some(error_cause.into()),
+                Some(Location::caller().to_string()),
+            )),
+        }
+    }
+
+    #[track_caller]
+    fn map_context<T: 'static + Display + Debug, F: FnOnce(&E) -> T>(
+        self,
+        op: F,
+    ) -> std::result::Result<O, ChainError<T>> {
+        match self {
+            Ok(t) => Ok(t),
+            Err(error_cause) => {
+                let kind = op(&error_cause);
+                Err(ChainError::new(
+                    kind,
+                    Some(error_cause.into()),
+                    Some(Location::caller().to_string()),
+                ))
+            }
+        }
+    }
+}
+
+/// An iterator over all error causes/sources
+pub struct ErrorIter<'a> {
     current: Option<&'a (dyn Error + 'static)>,
 }
 
@@ -657,40 +726,47 @@ impl<T: 'static + Display + Debug> Display for ChainError<T> {
 impl<T: 'static + Display + Debug> Debug for ChainError<T> {
     #[inline]
     fn fmt(&self, f: &mut Formatter<'_>) -> Result {
-        #[cfg(not(feature = "no-fileline"))]
-        {
-            if let Some(ref o) = self.occurrence {
-                Display::fmt(o, f)?;
-            }
-        }
+        if f.alternate() {
+            let mut f = f.debug_struct(&format!("ChainError<{}>", std::any::type_name::<T>()));
 
-        if self.is_chain::<String>() {
-            Display::fmt(&self.kind, f)?;
+            let f = f
+                .field("occurrence", &self.occurrence)
+                .field("kind", &self.kind)
+                .field("source", &self.source());
+
+            f.finish()
         } else {
-            Debug::fmt(&self.kind, f)?;
-        }
+            if let Some(ref o) = self.occurrence {
+                write!(f, "{}: ", o)?;
+            }
+
+            if TypeId::of::<String>() == TypeId::of::<T>()
+                || TypeId::of::<&str>() == TypeId::of::<T>()
+            {
+                Display::fmt(&self.kind, f)?;
+            } else {
+                Debug::fmt(&self.kind, f)?;
+            }
 
-        #[cfg(not(feature = "no-debug-cause"))]
-        {
             if let Some(e) = self.source() {
                 writeln!(f, "\nCaused by:")?;
                 Debug::fmt(&e, f)?;
             }
+            Ok(())
         }
-        Ok(())
     }
 }
 
 /// `ChainErrorFrom<T>` is similar to `From<T>`
 pub trait ChainErrorFrom<T>: Sized {
     /// similar to From<T>::from()
-    fn chain_error_from(from: T, line_filename: Option<&'static str>) -> ChainError<Self>;
+    fn chain_error_from(from: T, line_filename: Option<String>) -> ChainError<Self>;
 }
 
 /// `IntoChainError<T>` is similar to `Into<T>`
 pub trait IntoChainError<T>: Sized {
     /// similar to Into<T>::into()
-    fn into_chain_error(self, line_filename: Option<&'static str>) -> ChainError<T>;
+    fn into_chain_error(self, line_filename: Option<String>) -> ChainError<T>;
 }
 
 impl<T, U> IntoChainError<U> for T
@@ -698,7 +774,7 @@ where
     U: ChainErrorFrom<T>,
 {
     #[inline]
-    fn into_chain_error(self, line_filename: Option<&'static str>) -> ChainError<U> {
+    fn into_chain_error(self, line_filename: Option<String>) -> ChainError<U> {
         U::chain_error_from(self, line_filename)
     }
 }
@@ -709,297 +785,12 @@ where
     U: 'static + Display + Debug,
 {
     #[inline]
-    fn chain_error_from(t: T, line_filename: Option<&'static str>) -> ChainError<Self> {
+    fn chain_error_from(t: T, line_filename: Option<String>) -> ChainError<Self> {
         let e: U = t.into();
         ChainError::new(e, None, line_filename)
     }
 }
 
-/*
-impl<T, U> ChainErrorFrom<T> for U
-    where
-        T: 'static + Error + Into<Box<T>> + Clone,
-        U: 'static + Display + Debug + From<T>,
-{
-    #[inline]
-    fn chain_error_from(t: T, line_filename: Option<&'static str>) -> ChainError<Self> {
-        ChainError::new(U::from(t.clone()), Some(Box::from(t)), line_filename)
-    }
-}
-*/
-
-/// map into `ChainError<T>` with `T::from(err)`
-///
-/// adds `line!()` and `file!()` information
-#[macro_export]
-macro_rules! minto_cherr {
-    ( $k:ident ) => (
-        |e| $crate::cherr!(e, $k::from(&e))
-    );
-    ( $enum:ident $(:: $enum_path:ident)* ) => (
-        |e| $crate::cherr!(e, $enum $(:: $enum_path)*::from(&e))
-    );
-}
-
-/// Creates a new `ChainError<T>`
-///
-/// # Examples
-///
-/// Create a new ChainError<FooError>, where `FooError` must implement `Display` and `Debug`.
-/// ```rust
-/// # use chainerror::*;
-/// # #[derive(Debug)]
-/// enum FooError {
-///     Bar,
-///     Baz(&'static str),
-/// }
-/// # impl ::std::fmt::Display for FooError {
-/// #     fn fmt(&self, f: &mut ::std::fmt::Formatter) -> ::std::fmt::Result {
-/// #         match self {
-/// #             FooError::Bar => write!(f, "Bar Error"),
-/// #             FooError::Baz(s) => write!(f, "Baz Error: '{}'", s),
-/// #         }
-/// #     }
-/// # }
-///
-/// //  impl ::std::fmt::Display for FooError
-///
-/// fn do_some_stuff() -> bool {
-///     false
-/// }
-///
-/// fn func() -> ChainResult<(), FooError> {
-///     if ! do_some_stuff() {
-///         Err(cherr!(FooError::Baz("Error")))?;
-///     }
-///     Ok(())
-/// }
-/// # pub fn main() {
-/// #     match func().unwrap_err().kind() {
-/// #         FooError::Baz(s) if s == &"Error" => {}
-/// #         _ => panic!(),
-/// #     }
-/// # }
-/// ```
-///
-/// Additionally an error cause can be added.
-///
-/// ```rust
-/// # use chainerror::*;
-/// # use std::io;
-/// # use std::error::Error;
-/// # #[derive(Debug)]
-/// # enum FooError {
-/// #     Bar,
-/// #     Baz(&'static str),
-/// # }
-/// # impl ::std::fmt::Display for FooError {
-/// #     fn fmt(&self, f: &mut ::std::fmt::Formatter) -> ::std::fmt::Result {
-/// #         match self {
-/// #             FooError::Bar => write!(f, "Bar Error"),
-/// #             FooError::Baz(s) => write!(f, "Baz Error: '{}'", s),
-/// #         }
-/// #     }
-/// # }
-/// fn do_some_stuff() -> Result<(), Box<dyn Error + Send + Sync>> {
-///     Err(io::Error::from(io::ErrorKind::NotFound))?;
-///     Ok(())
-/// }
-///
-/// fn func() -> ChainResult<(), FooError> {
-///     do_some_stuff().map_err(
-///         |e| cherr!(e, FooError::Baz("Error"))
-///     )?;
-///     Ok(())
-/// }
-/// # pub fn main() {
-/// #     match func().unwrap_err().kind() {
-/// #         FooError::Baz(s) if s == &"Error" => {}
-/// #         _ => panic!(),
-/// #     }
-/// # }
-/// ```
-#[macro_export]
-macro_rules! cherr {
-    ( $k:expr ) => ({
-        $crate::ChainError::new($k, None, Some(concat!(file!(), ":", line!(), ": ")))
-    });
-    ( None, $k:expr ) => ({
-        $crate::ChainError::new($k, None, Some(concat!(file!(), ":", line!(), ": ")))
-    });
-    ( None, $fmt:expr, $($arg:tt)+ ) => ({
-        $crate::cherr!(None, format!($fmt, $($arg)+ ))
-    });
-    ( None, $fmt:expr, $($arg:tt)+ ) => ({
-        $crate::cherr!(None, format!($fmt, $($arg)+ ))
-    });
-    ( $e:path, $k:expr ) => ({
-        $crate::ChainError::new($k, Some(Box::from($e)), Some(concat!(file!(), ":", line!(), ": ")))
-    });
-    ( $e:path, $fmt:expr, $($arg:tt)+ ) => ({
-        $crate::cherr!($e, format!($fmt, $($arg)+ ))
-    });
-}
-
-/// Convenience macro for `|e| cherr!(e, format!(…))`
-///
-/// # Examples
-///
-/// ```rust
-/// # use crate::chainerror::*;
-/// # use std::error::Error;
-/// # use std::io;
-/// # use std::result::Result;
-/// # fn do_some_io() -> Result<(), Box<dyn Error + Send + Sync>> {
-/// #     Err(io::Error::from(io::ErrorKind::NotFound))?;
-/// #     Ok(())
-/// # }
-/// fn func2() -> Result<(), Box<dyn Error + Send + Sync>> {
-///     let filename = "foo.txt";
-///     do_some_io().map_err(mstrerr!("Error reading '{}'", filename))?;
-///     Ok(())
-/// }
-///
-/// fn func1() -> Result<(), Box<dyn Error + Send + Sync>> {
-///     func2().map_err(mstrerr!("func1 error"))?;
-///     Ok(())
-/// }
-///
-/// # fn main() {
-/// #     if let Err(e) = func1() {
-/// #         #[cfg(not(windows))]
-/// #         assert_eq!(
-/// #             format!("\n{:?}\n", e), r#"
-/// # src/lib.rs:18: func1 error
-/// # Caused by:
-/// # src/lib.rs:13: Error reading 'foo.txt'
-/// # Caused by:
-/// # Kind(NotFound)
-/// # "#
-/// #         );
-/// #     } else {
-/// #         unreachable!();
-/// #     }
-/// # }
-/// ```
-///
-/// `mstrerr!()` can also be used to map a new `ChainError<T>`, where T was defined with
-/// `derive_str_cherr!(T)`
-///
-/// ```rust
-/// # use crate::chainerror::*;
-/// # use std::error::Error;
-/// # use std::io;
-/// # use std::result::Result;
-/// # fn do_some_io() -> Result<(), Box<dyn Error + Send + Sync>> {
-/// #     Err(io::Error::from(io::ErrorKind::NotFound))?;
-/// #     Ok(())
-/// # }
-/// derive_str_cherr!(Func2Error);
-///
-/// fn func2() -> Result<(), Box<dyn Error + Send + Sync>> {
-///     let filename = "foo.txt";
-///     do_some_io().map_err(mstrerr!(Func2Error, "Error reading '{}'", filename))?;
-///     Ok(())
-/// }
-///
-/// derive_str_cherr!(Func1Error);
-///
-/// fn func1() -> Result<(), Box<dyn Error + Send + Sync>> {
-///     func2().map_err(mstrerr!(Func1Error, "func1 error"))?;
-///     Ok(())
-/// }
-/// # fn main() {
-/// #     if let Err(e) = func1() {
-/// #         if let Some(f1err) = e.downcast_chain_ref::<Func1Error>() {
-/// #             assert!(f1err.find_cause::<ChainError<Func2Error>>().is_some());
-/// #             assert!(f1err.find_chain_cause::<Func2Error>().is_some());
-/// #         } else {
-/// #             panic!();
-/// #         }
-/// #     } else {
-/// #         unreachable!();
-/// #     }
-/// # }
-/// ```
-#[macro_export]
-macro_rules! mstrerr {
-    ( $t:path, $msg:expr ) => ({
-        |e| $crate::cherr!(e, $t ($msg.to_string()))
-    });
-    ( $t:path, $msg:expr, ) => ({
-        |e| $crate::cherr!(e, $t ($msg.to_string()))
-    });
-    ( $t:path, $fmt:expr, $($arg:tt)+ ) => ({
-        |e| $crate::cherr!(e, $t (format!($fmt, $($arg)+ )))
-    });
-    ($msg:expr) => ({
-        |e| $crate::cherr!(e, $msg.to_string())
-    });
-    ($msg:expr, ) => ({
-        |e| $crate::cherr!(e, $msg.to_string())
-    });
-    ($fmt:expr, $($arg:tt)+) => ({
-        |e| $crate::cherr!(e, format!($fmt, $($arg)+ ))
-    });
-}
-
-/// Convenience macro for `cherr!(T(format!(…)))` where `T(String)`
-///
-/// # Examples
-///
-/// ```rust
-/// # use crate::chainerror::*;
-/// # use std::error::Error;
-/// # use std::result::Result;
-/// derive_str_cherr!(Func2Error);
-///
-/// fn func2() -> ChainResult<(), Func2Error> {
-///     let filename = "foo.txt";
-///     Err(strerr!(Func2Error, "Error reading '{}'", filename))
-/// }
-///
-/// derive_str_cherr!(Func1Error);
-///
-/// fn func1() -> Result<(), Box<dyn Error>> {
-///     func2().map_err(mstrerr!(Func1Error, "func1 error"))?;
-///     Ok(())
-/// }
-/// # fn main() {
-/// #     if let Err(e) = func1() {
-/// #         if let Some(f1err) = e.downcast_chain_ref::<Func1Error>() {
-/// #             assert!(f1err.find_cause::<ChainError<Func2Error>>().is_some());
-/// #             assert!(f1err.find_chain_cause::<Func2Error>().is_some());
-/// #         } else {
-/// #             panic!();
-/// #         }
-/// #     } else {
-/// #         unreachable!();
-/// #     }
-/// # }
-/// ```
-#[macro_export]
-macro_rules! strerr {
-    ( $t:path, $msg:expr ) => ({
-        $crate::cherr!($t ($msg.to_string()))
-    });
-    ( $t:path, $msg:expr, ) => ({
-        $crate::cherr!($t ($msg.to_string()))
-    });
-    ( $t:path, $fmt:expr, $($arg:tt)+ ) => ({
-        $crate::cherr!($t (format!($fmt, $($arg)+ )))
-    });
-    ($msg:expr) => ({
-        $crate::cherr!($msg.to_string())
-    });
-    ($msg:expr, ) => ({
-        $crate::cherr!($msg.to_string())
-    });
-    ($fmt:expr, $($arg:tt)+) => ({
-        $crate::cherr!(format!($fmt, $($arg)+ ))
-    });
-}
-
 /// Convenience macro to create a "new type" T(String) and implement Display + Debug for T
 ///
 /// # Examples
@@ -1013,21 +804,20 @@ macro_rules! strerr {
 /// #     Err(io::Error::from(io::ErrorKind::NotFound))?;
 /// #     Ok(())
 /// # }
-/// derive_str_cherr!(Func2Error);
+/// derive_str_context!(Func2Error);
 ///
 /// fn func2() -> ChainResult<(), Func2Error> {
 ///     let filename = "foo.txt";
-///     do_some_io().map_err(mstrerr!(Func2Error, "Error reading '{}'", filename))?;
+///     do_some_io().context(Func2Error(format!("Error reading '{}'", filename)))?;
 ///     Ok(())
 /// }
 ///
-/// derive_str_cherr!(Func1Error);
+/// derive_str_context!(Func1Error);
 ///
 /// fn func1() -> Result<(), Box<dyn Error>> {
-///     func2().map_err(mstrerr!(Func1Error, "func1 error"))?;
+///     func2().context(Func1Error("func1 error".into()))?;
 ///     Ok(())
 /// }
-/// # fn main() {
 /// #     if let Err(e) = func1() {
 /// #         if let Some(f1err) = e.downcast_chain_ref::<Func1Error>() {
 /// #             assert!(f1err.find_cause::<ChainError<Func2Error>>().is_some());
@@ -1038,10 +828,9 @@ macro_rules! strerr {
 /// #     } else {
 /// #         unreachable!();
 /// #     }
-/// # }
 /// ```
 #[macro_export]
-macro_rules! derive_str_cherr {
+macro_rules! derive_str_context {
     ($e:ident) => {
         #[derive(Clone)]
         pub struct $e(pub String);
@@ -1070,8 +859,8 @@ macro_rules! derive_str_cherr {
 /// # Examples
 ///
 /// ```rust
+/// use chainerror::prelude::v1::*;
 /// use std::io;
-/// use chainerror::*;
 ///
 /// fn do_some_io(_f: &str) -> std::result::Result<(), io::Error> {
 ///     return Err(io::Error::from(io::ErrorKind::NotFound));
@@ -1117,11 +906,9 @@ macro_rules! derive_str_cherr {
 /// pub fn func1() -> std::result::Result<(), Error> {
 ///     let filename = "bar.txt";
 ///
-///     do_some_io(filename)
-///         .map_err(|e| cherr!(e, ErrorKind::from_io_error(&e, filename.into())))?;
-///     do_some_io(filename).map_err(|e| cherr!(e, ErrorKind::IO(filename.into())))?;
-///     do_some_io(filename).map_err(|e| cherr!(e, ErrorKind::from(&e)))?;
-///     do_some_io(filename).map_err(minto_cherr!(ErrorKind))?;
+///     do_some_io(filename).map_context(|e| ErrorKind::from_io_error(e, filename.into()))?;
+///     do_some_io(filename).map_context(|e| ErrorKind::IO(filename.into()))?;
+///     do_some_io(filename).map_context(|e| ErrorKind::from(e))?;
 ///     Ok(())
 /// }
 /// ```
@@ -1157,19 +944,6 @@ macro_rules! derive_err_kind {
             }
         }
 
-        impl $crate::ChainErrorFrom<$e> for $k
-        where
-            $k: Clone,
-        {
-            #[inline]
-            fn chain_error_from(
-                t: $e,
-                line_filename: Option<&'static str>,
-            ) -> $crate::ChainError<$k> {
-                $crate::ChainError::new((*t.kind()).clone(), Some(Box::from(t)), line_filename)
-            }
-        }
-
         impl std::error::Error for $e {
             fn source(&self) -> Option<&(dyn std::error::Error + 'static)> {
                 self.0.source()
